from torch import nn
from torch.autograd import Function

# from .libs import bottom_pool, right_pool, left_pool, top_pool
# from cornerpool import libs
import cornerpool.libs.top_pool
import cornerpool.libs.bottom_pool
import cornerpool.libs.right_pool
import cornerpool.libs.left_pool


__all__ = ["TopPool", "BottomPool", "LeftPool", "RightPool"]


class TopPoolFunction(Function):
    @staticmethod
    def forward(ctx, input):
        output = cornerpool.libs.top_pool.forward(input)[0]
        ctx.save_for_backward(input)
        return output

    @staticmethod
    def backward(ctx, grad_output):
        input = ctx.saved_variables[0]
        output = cornerpool.libs.top_pool.backward(input, grad_output)[0]
        return output


class BottomPoolFunction(Function):
    @staticmethod
    def forward(ctx, input):
        output = cornerpool.libs.bottom_pool.forward(input)[0]
        ctx.save_for_backward(input)
        return output

    @staticmethod
    def backward(ctx, grad_output):
        input = ctx.saved_variables[0]
        output = cornerpool.libs.bottom_pool.backward(input, grad_output)[0]
        return output


class LeftPoolFunction(Function):
    @staticmethod
    def forward(ctx, input):
        output = cornerpool.libs.left_pool.forward(input)[0]
        ctx.save_for_backward(input)
        return output

    @staticmethod
    def backward(ctx, grad_output):
        input = ctx.saved_variables[0]
        output = cornerpool.libs.left_pool.backward(input, grad_output)[0]
        return output


class RightPoolFunction(Function):
    @staticmethod
    def forward(ctx, input):
        output = cornerpool.libs.right_pool.forward(input)[0]
        ctx.save_for_backward(input)
        return output

    @staticmethod
    def backward(ctx, grad_output):
        input = ctx.saved_variables[0]
        output = cornerpool.libs.right_pool.backward(input, grad_output)[0]
        return output


class TopPool(nn.Module):
    def forward(self, x):
        return TopPoolFunction.apply(x)


class BottomPool(nn.Module):
    def forward(self, x):
        return BottomPoolFunction.apply(x)


class LeftPool(nn.Module):
    def forward(self, x):
        return LeftPoolFunction.apply(x)


class RightPool(nn.Module):
    def forward(self, x):
        return RightPoolFunction.apply(x)



from os import path

from setuptools import find_packages, setup
from torch.utils.cpp_extension import BuildExtension, CppExtension

here = path.abspath(path.dirname(__file__))
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="cornerpool",
    version="0.0.2",
    description="Cornerpooling in pytorch",
    long_description=long_description,
    author="Bishwarup Bhattacharjee",
    author_email="write2bishwarup@gmail.com",
    python_requires=">=3.6",
    packages=find_packages(),
    ext_modules=[
        CppExtension("cornerpool.libs.top_pool", ["./cornerpool/src/top_pool.cpp"]),
        CppExtension(
            "cornerpool.libs.bottom_pool", ["./cornerpool/src/bottom_pool.cpp"]
        ),
        CppExtension("cornerpool.libs.left_pool", ["./cornerpool/src/left_pool.cpp"]),
        CppExtension("cornerpool.libs.right_pool", ["./cornerpool/src/right_pool.cpp"]),
    ],
    cmdclass={"build_ext": BuildExtension},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    url="https://github.com/bishwarup307/cornerpool",
)
